(function() {
	
	"use strict";
	
	var menuItem;
	var winMenuReloadFromDisk;
	
	EDITOR.plugin({
		desc: "Adds option to reload the file from disk in the context menu",
		load: loadFileReload,
		unload: unloadFileReload
	});
	
	function loadFileReload() {
		
		menuItem = EDITOR.ctxMenu.add(S("reload_from_disk"), reloadFile, 6);
		winMenuReloadFromDisk = EDITOR.windowMenu.add(S("reload_from_disk"), [S("Edit"), 4], reloadFile);
		
		EDITOR.registerAltKey({char: "back", alt:1, label: S("reload_from_disk"), fun: reloadFile});
		
	}
	
	function unloadFileReload() {
		EDITOR.ctxMenu.remove(menuItem);
		EDITOR.windowMenu.remove(winMenuReloadFromDisk);
		EDITOR.unregisterAltKey(reloadFile);
	}
	
	function reloadFile() {
		var file = EDITOR.currentFile;
		
if(!file) return true;

		if(file.changed) {
			
			var yes = "Yes, discard changes";
			var no = "NO"
			var msg = "";
			var text = file.text;
			
			msg = "Are you sure you want to reload the file from disk ?<br>" + file.path;
			
			confirmBox(msg, [yes, no], function (answer) {
				if(answer == yes) {
					reload();
				}
			});
		}
		else reload();
		
		EDITOR.ctxMenu.hide();
		
		function reload() {
			
			if(file.isBig) {
				var filePath = file.path;
				EDITOR.closeFile(file);
				EDITOR.openFile(filePath);
			}
			else {
			EDITOR.readFromDisk(file.path, function(err, path, text, hash) {
				if(err) {
					if(err.code == "ENOENT") {
						alertBox("The file no longer exist on disk: " + file.path);
					}
					else throw err;
				}
				else {
					file.reload(text);
					file.hash = hash;
					file.saved(); // Because we reloaded from disk
				}
				});
		}
		}
		
	}
	
	
})();